﻿using System;
using Ozeki.Media;

using System.Threading;
using System.IO;

namespace _02_MediaHandler_Example
{
    class Program
    {
        static void Main(string[] args)
        {

            Console.WriteLine("This application show you how to connect the media handlers.");
            Console.WriteLine("");

            Console.WriteLine("1: Connect the microphone to the speaker");
            Console.WriteLine("2: Play mp3 file with the speaker");
            Console.WriteLine("3: 1 and 2 at same time");
            Console.WriteLine("4: Read a text");
            Console.WriteLine("5: Record voice and save that to a wav file");
            Console.WriteLine("6: Record a webcamera view and voice and save that to an mp4 file");

            Console.WriteLine("");
            Console.WriteLine("Please give me the number, which you choose:");

            var param = Console.ReadLine();

            Console.WriteLine("");

            switch (param)
            {
                case "1":
                    MicrophoneToSpeaker();
                    break;

                case "2":
                    MP3ToSpeaker();
                    break;

                case "3":
                    MicrophoneAndMP3ToSpeaker();
                    break;

                case "4":
                    TextToSpeaker();
                    break;

                case "5":
                    RecordVoice();
                    break;

                case "6":
                    CameraToMPEG4();
                    break;

                default:
                    Console.WriteLine("The selected number is not valid.");
                    Console.ReadLine();
                    break;
            }

        }


        static void MicrophoneToSpeaker()
        {
            Microphone microphone = Microphone.GetDefaultDevice();
            Speaker speaker = Speaker.GetDefaultDevice();

            MediaConnector mediaConnector = new MediaConnector();

            if (microphone != null && speaker != null)
            {
                mediaConnector.Connect(microphone, speaker);

                microphone.Start();
                speaker.Start();

                Console.WriteLine("Please, press enter to exit the program.");
                Console.ReadLine();

                mediaConnector.Dispose();
                microphone.Dispose();
                speaker.Dispose();
            }

            else
            {
                Console.WriteLine("The microphone or the speaker are not available.");
                Console.WriteLine("Please, press enter to exit the program.");
                Console.ReadLine();
            }
        }

        static void MP3ToSpeaker()
        {
            try
            {
                MP3StreamPlayback mp3Player = new MP3StreamPlayback("../../Resources/test.mp3");
                Speaker speaker = Speaker.GetDefaultDevice();

                MediaConnector mediaConnector = new MediaConnector();

                DirectoryInfo di = new DirectoryInfo(Directory.GetCurrentDirectory());

                Console.WriteLine("The directory where the mp3 is found: " + di.Parent.Parent.FullName + "\\Resources\\test.mp3");
                Console.WriteLine("");

                if (mp3Player != null && speaker != null)
                {
                    mediaConnector.Connect(mp3Player, speaker);

                    mp3Player.Start();
                    speaker.Start();


                    Console.WriteLine("Please, press enter to exit the program.");
                    Console.ReadLine();

                    mediaConnector.Dispose();
                    mp3Player.Dispose();
                    speaker.Dispose();
                }

                else
                {
                    Console.WriteLine("The speaker is not available.");
                    Console.WriteLine("Please, press enter to exit the program.");
                    Console.ReadLine();
                }
            }

            catch (Exception exception)
            {
                Console.WriteLine("The mp3 file is not available.");
                Console.WriteLine("Please, press enter to exit the program.");
                Console.ReadLine();
            }
        }

        static void MicrophoneAndMP3ToSpeaker()
        {
            Microphone microphone = Microphone.GetDefaultDevice();
            Speaker speaker = Speaker.GetDefaultDevice();
            try
            {
                MP3StreamPlayback mp3Player = new MP3StreamPlayback("../../Resources/test.mp3");
                MediaConnector mediaConnector = new MediaConnector();

                DirectoryInfo di = new DirectoryInfo(Directory.GetCurrentDirectory());

                Console.WriteLine("The directory where the mp3 is found: " + di.Parent.Parent.FullName + "\\Resources\\test.mp3");
                Console.WriteLine("");

                if (microphone != null && mp3Player != null && speaker != null)
                {
                    mediaConnector.Connect(microphone, speaker);
                    mediaConnector.Connect(mp3Player, speaker);

                    microphone.Start();
                    mp3Player.Start();
                    speaker.Start();

                    Console.WriteLine("Please, press enter to exit the program.");
                    Console.ReadLine();

                    mediaConnector.Dispose();
                    microphone.Dispose();
                    mp3Player.Dispose();
                    speaker.Dispose();
                }

                else
                {
                    Console.WriteLine("The microphone or the speaker are not available.");
                    Console.WriteLine("Please, press enter to exit the program.");
                    Console.ReadLine();
                }
            }

            catch (Exception exception)
            {
                Console.WriteLine("The mp3 file is not available.");
                Console.WriteLine("Please, press enter to exit the program.");
                Console.ReadLine();
            }
        }

        static void TextToSpeaker()
        {
            Speaker speaker = Speaker.GetDefaultDevice();
            TextToSpeech textToSpeech = new TextToSpeech();

            MediaConnector mediaConnector = new MediaConnector();

            if (speaker != null)
            {
                mediaConnector.Connect(textToSpeech, speaker);

                textToSpeech.AddAndStartText("Please read this text.");

                textToSpeech.Start();
                speaker.Start();

                Console.WriteLine("Please, press enter to exit the program.");
                Console.ReadLine();

                mediaConnector.Dispose();
                textToSpeech.Dispose();
                speaker.Dispose();
            }

            else
            {
                Console.WriteLine("The speaker is not available.");
                Console.WriteLine("Please, press enter to exit the program.");
                Console.ReadLine();
            }
        }

        static void CameraToMPEG4()
        {
            Microphone microphone = Microphone.GetDefaultDevice();
            Ozeki.Camera.WebCamera webCamera = new Ozeki.Camera.WebCamera();

            MediaConnector mediaConnector = new MediaConnector();

            if (microphone != null && webCamera != null)
            {
                MPEG4Recorder mpeg4Recorder = new MPEG4Recorder("../../Resources/test.mp4");

                mediaConnector.Connect(microphone, mpeg4Recorder.AudioRecorder);
                mediaConnector.Connect(webCamera, mpeg4Recorder.VideoRecorder);

                microphone.Start();
                webCamera.Start();

                Console.WriteLine("Mpeg4 recording started.");

                Thread.Sleep(5000);

                Console.WriteLine("Multiplexing in progress.");

                mediaConnector.Dispose();
                webCamera.Dispose();
                microphone.Dispose();

                mpeg4Recorder.MultiplexFinished += (sender, e) =>
                {
                    mpeg4Recorder.Dispose();
                    Console.WriteLine("The mpeg4 is ready.");
                    DirectoryInfo di = new DirectoryInfo(Directory.GetCurrentDirectory());

                    Console.WriteLine("The directory where the mp4 will be found: " + di.Parent.Parent.FullName + "\\Resources\\test.mp4");
                    Console.WriteLine("");
                    Console.WriteLine("Please, press enter to exit the program.");
                };

                mpeg4Recorder.Multiplex();

                Console.ReadLine();
            }

            else
            {
                Console.WriteLine("The microphone or the webcamera are not available.");
                Console.WriteLine("Please, press enter to exit the program.");
                Console.ReadLine();
            }
        }

        static void RecordVoice()
        {
            Microphone microphone = Microphone.GetDefaultDevice();
            MediaConnector mediaConnector = new MediaConnector();

            if (microphone != null)
            {
                microphone.Start();

                string filename = "recorded-voice" + "-" + DateTime.Now.Hour.ToString() + "-" + DateTime.Now.Minute.ToString() + "-" +
                                  DateTime.Now.Second.ToString() + ".wav";
                var recorder = new WaveStreamRecorder("../../Resources/" + filename);

                mediaConnector.Connect(microphone, recorder); // connects the voices to the recorder
                recorder.Start();  // starts the recording

                Console.WriteLine("Wav recording started.\n");

                DirectoryInfo di = new DirectoryInfo(Directory.GetCurrentDirectory());
                Console.WriteLine("The directory where the wav file will be found: " + di.Parent.Parent.FullName + "\\Resources\\" + filename);

                Console.WriteLine("Please, press enter to exit the program.");
                Console.ReadLine();

                recorder.Stop();
                microphone.Stop();
                mediaConnector.Dispose();
            }

            else
            {
                Console.WriteLine("The microphone is not available.");
                Console.WriteLine("Please, press enter to exit the program.");
                Console.ReadLine();
            }
        }
    }
}
